@@ -21,6 +21,7 @@
 #include <pulsar/ClientConfiguration.h>
 #include <pulsar/Version.h>
 
+#include <chrono>
 #include <random>
 #include <sstream>
 
@@ -78,8 +79,8 @@ typedef std::vector<std::string> StringList;
 ClientImpl::ClientImpl(const std::string& serviceUrl, const ClientConfiguration& clientConfiguration)
     : mutex_(),
       state_(Open),
-      serviceNameResolver_(serviceUrl),
-      clientConfiguration_(ClientConfiguration(clientConfiguration).setUseTls(serviceNameResolver_.useTls())),
+      clientConfiguration_(ClientConfiguration(clientConfiguration)
+                               .setUseTls(ServiceNameResolver::useTls(ServiceURI(serviceUrl)))),
       memoryLimitController_(clientConfiguration.getMemoryLimit()),
       ioExecutorProvider_(std::make_shared<ExecutorServiceProvider>(clientConfiguration_.getIOThreads())),
       listenerExecutorProvider_(
@@ -90,32 +91,35 @@ ClientImpl::ClientImpl(const std::string& serviceUrl, const ClientConfiguration&
             ClientImpl::getClientVersion(clientConfiguration)),
       producerIdGenerator_(0),
       consumerIdGenerator_(0),
-      closingError(ResultOk) {
+      closingError(ResultOk),
+      useProxy_(false),
+      lookupCount_(0L) {
     std::unique_ptr<LoggerFactory> loggerFactory = clientConfiguration_.impl_->takeLogger();
-    if (!loggerFactory) {
-        // Use default simple console logger
-        loggerFactory.reset(new ConsoleLoggerFactory);
+    if (loggerFactory) {
+        LogUtils::setLoggerFactory(std::move(loggerFactory));
     }
-    LogUtils::setLoggerFactory(std::move(loggerFactory));
-
-    LookupServicePtr underlyingLookupServicePtr;
-    if (serviceNameResolver_.useHttp()) {
-        LOG_DEBUG("Using HTTP Lookup");
-        underlyingLookupServicePtr = std::make_shared<HTTPLookupService>(
-            std::ref(serviceNameResolver_), std::cref(clientConfiguration_),
-            std::cref(clientConfiguration_.getAuthPtr()));
-    } else {
-        LOG_DEBUG("Using Binary Lookup");
-        underlyingLookupServicePtr = std::make_shared<BinaryProtoLookupService>(
-            std::ref(serviceNameResolver_), std::ref(pool_), std::cref(clientConfiguration_));
-    }
-
-    lookupServicePtr_ = RetryableLookupService::create(
-        underlyingLookupServicePtr, clientConfiguration_.getOperationTimeoutSeconds(), ioExecutorProvider_);
+    lookupServicePtr_ = createLookup(serviceUrl);
 }
 
 ClientImpl::~ClientImpl() { shutdown(); }
 
+LookupServicePtr ClientImpl::createLookup(const std::string& serviceUrl) {
+    LookupServicePtr underlyingLookupServicePtr;
+    if (ServiceNameResolver::useHttp(ServiceURI(serviceUrl))) {
+        LOG_DEBUG("Using HTTP Lookup");
+        underlyingLookupServicePtr = std::make_shared<HTTPLookupService>(
+            serviceUrl, std::cref(clientConfiguration_), std::cref(clientConfiguration_.getAuthPtr()));
+    } else {
+        LOG_DEBUG("Using Binary Lookup");
+        underlyingLookupServicePtr = std::make_shared<BinaryProtoLookupService>(
+            serviceUrl, std::ref(pool_), std::cref(clientConfiguration_));
+    }
+
+    auto lookupServicePtr = RetryableLookupService::create(
+        underlyingLookupServicePtr, clientConfiguration_.impl_->operationTimeout, ioExecutorProvider_);
+    return lookupServicePtr;
+}
+
 const ClientConfiguration& ClientImpl::conf() const { return clientConfiguration_; }
 
 MemoryLimitController& ClientImpl::getMemoryLimitController() { return memoryLimitController_; }
@@ -128,7 +132,21 @@ ExecutorServiceProviderPtr ClientImpl::getPartitionListenerExecutorProvider() {
     return partitionListenerExecutorProvider_;
 }
 
-LookupServicePtr ClientImpl::getLookup() { return lookupServicePtr_; }
+LookupServicePtr ClientImpl::getLookup(const std::string& redirectedClusterURI) {
+    if (redirectedClusterURI.empty()) {
+        return lookupServicePtr_;
+    }
+
+    Lock lock(mutex_);
+    auto it = redirectedClusterLookupServicePtrs_.find(redirectedClusterURI);
+    if (it == redirectedClusterLookupServicePtrs_.end()) {
+        auto lookup = createLookup(redirectedClusterURI);
+        redirectedClusterLookupServicePtrs_.emplace(redirectedClusterURI, lookup);
+        return lookup;
+    }
+
+    return it->second;
+}
 
 void ClientImpl::createProducerAsync(const std::string& topic, ProducerConfiguration conf,
                                      CreateProducerCallback callback, bool autoDownloadSchema) {
@@ -516,7 +534,8 @@ void ClientImpl::handleConsumerCreated(Result result, ConsumerImplBaseWeakPtr co
     }
 }
 
-Future<Result, ClientConnectionPtr> ClientImpl::getConnection(const std::string& topic, size_t key) {
+GetConnectionFuture ClientImpl::getConnection(const std::string& redirectedClusterURI,
+                                              const std::string& topic, size_t key) {
     Promise<Result, ClientConnectionPtr> promise;
 
     const auto topicNamePtr = TopicName::get(topic);
@@ -527,12 +546,15 @@ Future<Result, ClientConnectionPtr> ClientImpl::getConnection(const std::string&
     }
 
     auto self = shared_from_this();
-    lookupServicePtr_->getBroker(*topicNamePtr)
+    getLookup(redirectedClusterURI)
+        ->getBroker(*topicNamePtr)
         .addListener([this, self, promise, key](Result result, const LookupService::LookupResult& data) {
             if (result != ResultOk) {
                 promise.setFailed(result);
                 return;
             }
+            useProxy_ = data.proxyThroughServiceUrl;
+            lookupCount_++;
             pool_.getConnectionAsync(data.logicalAddress, data.physicalAddress, key)
                 .addListener([promise](Result result, const ClientConnectionWeakPtr& weakCnx) {
                     if (result == ResultOk) {
@@ -551,6 +573,35 @@ Future<Result, ClientConnectionPtr> ClientImpl::getConnection(const std::string&
     return promise.getFuture();
 }
 
+const std::string& ClientImpl::getPhysicalAddress(const std::string& redirectedClusterURI,
+                                                  const std::string& logicalAddress) {
+    if (useProxy_) {
+        return getLookup(redirectedClusterURI)->getServiceNameResolver().resolveHost();
+    } else {
+        return logicalAddress;
+    }
+}
+
+GetConnectionFuture ClientImpl::connect(const std::string& redirectedClusterURI,
+                                        const std::string& logicalAddress, size_t key) {
+    const auto& physicalAddress = getPhysicalAddress(redirectedClusterURI, logicalAddress);
+    Promise<Result, ClientConnectionPtr> promise;
+    pool_.getConnectionAsync(logicalAddress, physicalAddress, key)
+        .addListener([promise](Result result, const ClientConnectionWeakPtr& weakCnx) {
+            if (result == ResultOk) {
+                auto cnx = weakCnx.lock();
+                if (cnx) {
+                    promise.setValue(cnx);
+                } else {
+                    promise.setFailed(ResultConnectError);
+                }
+            } else {
+                promise.setFailed(result);
+            }
+        });
+    return promise.getFuture();
+}
+
 void ClientImpl::handleGetPartitions(const Result result, const LookupDataResultPtr partitionMetadata,
                                      TopicNamePtr topicName, GetPartitionsCallback callback) {
     if (result != ResultOk) {
@@ -603,6 +654,9 @@ void ClientImpl::closeAsync(CloseCallback callback) {
 
     memoryLimitController_.close();
     lookupServicePtr_->close();
+    for (const auto& it : redirectedClusterLookupServicePtrs_) {
+        it.second->close();
+    }
 
     auto producers = producers_.move();
     auto consumers = consumers_.move();
@@ -636,6 +690,7 @@ void ClientImpl::closeAsync(CloseCallback callback) {
     if (*numberOfOpenHandlers == 0 && callback) {
         handleClose(ResultOk, numberOfOpenHandlers, callback);
     }
+    lookupCount_ = 0;
 }
 
 void ClientImpl::handleClose(Result result, SharedInt numberOfOpenHandlers, ResultCallback callback) {
@@ -723,6 +778,7 @@ void ClientImpl::shutdown() {
     partitionListenerExecutorProvider_->close(timeoutProcessor.getLeftTimeout());
     timeoutProcessor.tok();
     LOG_DEBUG("partitionListenerExecutorProvider_ is closed");
+    lookupCount_ = 0;
 }
 
 uint64_t ClientImpl::newProducerId() {
@@ -770,4 +826,8 @@ std::string ClientImpl::getClientVersion(const ClientConfiguration& clientConfig
     return oss.str();
 }
 
+std::chrono::nanoseconds ClientImpl::getOperationTimeout(const ClientConfiguration& clientConfiguration) {
+    return clientConfiguration.impl_->operationTimeout;
+}
+
 } /* namespace pulsar */
