@@ -26,6 +26,7 @@
 #include "AckGroupingTracker.h"
 #include "AckGroupingTrackerDisabled.h"
 #include "AckGroupingTrackerEnabled.h"
+#include "AsioDefines.h"
 #include "BatchMessageAcker.h"
 #include "BatchedMessageIdImpl.h"
 #include "BitSet.h"
@@ -55,6 +56,9 @@ namespace pulsar {
 
 DECLARE_LOG_OBJECT()
 
+using std::chrono::milliseconds;
+using std::chrono::seconds;
+
 ConsumerImpl::ConsumerImpl(const ClientImplPtr client, const std::string& topic,
                            const std::string& subscriptionName, const ConsumerConfiguration& conf,
                            bool isPersistent, const ConsumerInterceptorsPtr& interceptors,
@@ -83,7 +87,6 @@ ConsumerImpl::ConsumerImpl(const ClientImplPtr client, const std::string& topic,
       availablePermits_(0),
       receiverQueueRefillThreshold_(config_.getReceiverQueueSize() / 2),
       consumerId_(client->newConsumerId()),
-      consumerName_(config_.getConsumerName()),
       consumerStr_("[" + topic + ", " + subscriptionName + ", " + std::to_string(consumerId_) + "] "),
       messageListenerRunning_(true),
       negativeAcksTracker_(std::make_shared<NegativeAcksTracker>(client, *this, conf)),
@@ -233,30 +236,29 @@ Future<Result, bool> ConsumerImpl::connectionOpened(const ClientConnectionPtr& c
     // sending the subscribe request.
     cnx->registerConsumer(consumerId_, get_shared_this_ptr());
 
-    if (duringSeek_) {
+    if (duringSeek()) {
         ackGroupingTrackerPtr_->flushAndClean();
     }
 
     Lock lockForMessageId(mutexForMessageId_);
-    // Update startMessageId so that we can discard messages after delivery restarts
-    const auto startMessageId = clearReceiveQueue();
+    clearReceiveQueue();
     const auto subscribeMessageId =
-        (subscriptionMode_ == Commands::SubscriptionModeNonDurable) ? startMessageId : boost::none;
-    startMessageId_ = startMessageId;
+        (subscriptionMode_ == Commands::SubscriptionModeNonDurable) ? startMessageId_.get() : boost::none;
     lockForMessageId.unlock();
 
     unAckedMessageTrackerPtr_->clear();
 
     ClientImplPtr client = client_.lock();
-    uint64_t requestId = client->newRequestId();
+    long requestId = client->newRequestId();
     SharedBuffer cmd = Commands::newSubscribe(
-        topic(), subscription_, consumerId_, requestId, getSubType(), consumerName_, subscriptionMode_,
+        topic(), subscription_, consumerId_, requestId, getSubType(), getConsumerName(), subscriptionMode_,
         subscribeMessageId, readCompacted_, config_.getProperties(), config_.getSubscriptionProperties(),
         config_.getSchema(), getInitialPosition(), config_.isReplicateSubscriptionStateEnabled(),
         config_.getKeySharedPolicy(), config_.getPriorityLevel());
 
     // Keep a reference to ensure object is kept alive.
     auto self = get_shared_this_ptr();
+    setFirstRequestIdAfterConnect(requestId);
     cnx->sendRequestWithId(cmd, requestId)
         .addListener([this, self, cnx, promise](Result result, const ResponseData& responseData) {
             Result handleResult = handleCreateConsumer(cnx, result);
@@ -274,7 +276,7 @@ void ConsumerImpl::connectionFailed(Result result) {
     // Keep a reference to ensure object is kept alive
     auto ptr = get_shared_this_ptr();
 
-    if (consumerCreatedPromise_.setFailed(result)) {
+    if (!isResultRetryable(result) && consumerCreatedPromise_.setFailed(result)) {
         state_ = Failed;
     }
 }
@@ -403,10 +405,9 @@ void ConsumerImpl::discardChunkMessages(std::string uuid, MessageId messageId, b
 }
 
 void ConsumerImpl::triggerCheckExpiredChunkedTimer() {
-    checkExpiredChunkedTimer_->expires_from_now(
-        boost::posix_time::milliseconds(expireTimeOfIncompleteChunkedMessageMs_));
+    checkExpiredChunkedTimer_->expires_from_now(milliseconds(expireTimeOfIncompleteChunkedMessageMs_));
     std::weak_ptr<ConsumerImplBase> weakSelf{shared_from_this()};
-    checkExpiredChunkedTimer_->async_wait([this, weakSelf](const boost::system::error_code& ec) -> void {
+    checkExpiredChunkedTimer_->async_wait([this, weakSelf](const ASIO_ERROR& ec) -> void {
         auto self = weakSelf.lock();
         if (!self) {
             return;
@@ -1045,14 +1046,23 @@ void ConsumerImpl::messageProcessed(Message& msg, bool track) {
  * Clear the internal receiver queue and returns the message id of what was the 1st message in the queue that
  * was
  * not seen by the application
+ * `startMessageId_` is updated so that we can discard messages after delivery restarts.
  */
-boost::optional<MessageId> ConsumerImpl::clearReceiveQueue() {
-    bool expectedDuringSeek = true;
-    if (duringSeek_.compare_exchange_strong(expectedDuringSeek, false)) {
-        return seekMessageId_.get();
+void ConsumerImpl::clearReceiveQueue() {
+    if (duringSeek()) {
+        if (!hasSoughtByTimestamp_.load(std::memory_order_acquire)) {
+            startMessageId_ = seekMessageId_.get();
+        }
+        SeekStatus expected = SeekStatus::COMPLETED;
+        if (seekStatus_.compare_exchange_strong(expected, SeekStatus::NOT_STARTED)) {
+            auto seekCallback = seekCallback_.release();
+            executor_->postWork([seekCallback] { seekCallback(ResultOk); });
+        }
+        return;
     } else if (subscriptionMode_ == Commands::SubscriptionModeDurable) {
-        return startMessageId_.get();
+        return;
     }
+
     Message nextMessageInQueue;
     if (incomingMessages_.peekAndClear(nextMessageInQueue)) {
         // There was at least one message pending in the queue
@@ -1068,16 +1078,12 @@ boost::optional<MessageId> ConsumerImpl::clearReceiveQueue() {
                                            .ledgerId(nextMessageId.ledgerId())
                                            .entryId(nextMessageId.entryId() - 1)
                                            .build();
-        return previousMessageId;
+        startMessageId_ = previousMessageId;
     } else if (lastDequedMessageId_ != MessageId::earliest()) {
         // If the queue was empty we need to restart from the message just after the last one that has been
         // dequeued
         // in the past
-        return lastDequedMessageId_;
-    } else {
-        // No message was received or dequeued by this consumer. Next message would still be the
-        // startMessageId
-        return startMessageId_.get();
+        startMessageId_ = lastDequedMessageId_;
     }
 }
 
@@ -1232,10 +1238,13 @@ void ConsumerImpl::negativeAcknowledge(const MessageId& messageId) {
     negativeAcksTracker_->add(messageId);
 }
 
-void ConsumerImpl::disconnectConsumer() {
-    LOG_INFO("Broker notification of Closed consumer: " << consumerId_);
+void ConsumerImpl::disconnectConsumer() { disconnectConsumer(boost::none); }
+
+void ConsumerImpl::disconnectConsumer(const boost::optional<std::string>& assignedBrokerUrl) {
+    LOG_INFO("Broker notification of Closed consumer: "
+             << consumerId_ << (assignedBrokerUrl ? (" assignedBrokerUrl: " + assignedBrokerUrl.get()) : ""));
     resetCnx();
-    scheduleReconnection();
+    scheduleReconnection(assignedBrokerUrl);
 }
 
 void ConsumerImpl::closeAsync(ResultCallback originalCallback) {
@@ -1469,7 +1478,7 @@ void ConsumerImpl::seekAsync(const MessageId& msgId, ResultCallback callback) {
         return;
     }
     const auto requestId = client->newRequestId();
-    seekAsyncInternal(requestId, Commands::newSeek(consumerId_, requestId, msgId), msgId, 0L, callback);
+    seekAsyncInternal(requestId, Commands::newSeek(consumerId_, requestId, msgId), SeekArg{msgId}, callback);
 }
 
 void ConsumerImpl::seekAsync(uint64_t timestamp, ResultCallback callback) {
@@ -1488,49 +1497,62 @@ void ConsumerImpl::seekAsync(uint64_t timestamp, ResultCallback callback) {
         return;
     }
     const auto requestId = client->newRequestId();
-    seekAsyncInternal(requestId, Commands::newSeek(consumerId_, requestId, timestamp), MessageId::earliest(),
-                      timestamp, callback);
+    seekAsyncInternal(requestId, Commands::newSeek(consumerId_, requestId, timestamp), SeekArg{timestamp},
+                      callback);
 }
 
 bool ConsumerImpl::isReadCompacted() { return readCompacted_; }
 
-inline bool hasMoreMessages(const MessageId& lastMessageIdInBroker, const MessageId& messageId) {
-    return lastMessageIdInBroker > messageId && lastMessageIdInBroker.entryId() != -1;
-}
-
 void ConsumerImpl::hasMessageAvailableAsync(HasMessageAvailableCallback callback) {
-    const auto startMessageId = startMessageId_.get();
-    Lock lock(mutexForMessageId_);
-    const auto messageId =
-        (lastDequedMessageId_ == MessageId::earliest()) ? startMessageId.value() : lastDequedMessageId_;
-
-    if (messageId == MessageId::latest()) {
-        lock.unlock();
-        getLastMessageIdAsync([callback](Result result, const GetLastMessageIdResponse& response) {
+    bool compareMarkDeletePosition;
+    {
+        std::lock_guard<std::mutex> lock{mutexForMessageId_};
+        compareMarkDeletePosition =
+            (lastDequedMessageId_ == MessageId::earliest()) &&
+            (startMessageId_.get().value_or(MessageId::earliest()) == MessageId::latest());
+    }
+    if (compareMarkDeletePosition || hasSoughtByTimestamp_.load(std::memory_order_acquire)) {
+        auto self = get_shared_this_ptr();
+        getLastMessageIdAsync([self, callback](Result result, const GetLastMessageIdResponse& response) {
             if (result != ResultOk) {
                 callback(result, {});
                 return;
             }
-            if (response.hasMarkDeletePosition() && response.getLastMessageId().entryId() >= 0) {
-                // We only care about comparing ledger ids and entry ids as mark delete position doesn't have
-                // other ids such as batch index
-                callback(ResultOk, compareLedgerAndEntryId(response.getMarkDeletePosition(),
-                                                           response.getLastMessageId()) < 0);
+            auto handleResponse = [self, response, callback] {
+                if (response.hasMarkDeletePosition() && response.getLastMessageId().entryId() >= 0) {
+                    // We only care about comparing ledger ids and entry ids as mark delete position
+                    // doesn't have other ids such as batch index
+                    auto compareResult = compareLedgerAndEntryId(response.getMarkDeletePosition(),
+                                                                 response.getLastMessageId());
+                    callback(ResultOk, self->config_.isStartMessageIdInclusive() ? compareResult <= 0
+                                                                                 : compareResult < 0);
+                } else {
+                    callback(ResultOk, false);
+                }
+            };
+            if (self->config_.isStartMessageIdInclusive() &&
+                !self->hasSoughtByTimestamp_.load(std::memory_order_acquire)) {
+                self->seekAsync(response.getLastMessageId(), [callback, handleResponse](Result result) {
+                    if (result != ResultOk) {
+                        callback(result, {});
+                        return;
+                    }
+                    handleResponse();
+                });
             } else {
-                callback(ResultOk, false);
+                handleResponse();
             }
         });
     } else {
-        if (hasMoreMessages(lastMessageIdInBroker_, messageId)) {
-            lock.unlock();
+        if (hasMoreMessages()) {
             callback(ResultOk, true);
             return;
         }
-        lock.unlock();
-
-        getLastMessageIdAsync([callback, messageId](Result result, const GetLastMessageIdResponse& response) {
-            callback(result, (result == ResultOk) && hasMoreMessages(response.getLastMessageId(), messageId));
-        });
+        auto self = get_shared_this_ptr();
+        getLastMessageIdAsync(
+            [this, self, callback](Result result, const GetLastMessageIdResponse& response) {
+                callback(result, (result == ResultOk) && hasMoreMessages());
+            });
     }
 }
 
@@ -1582,7 +1604,7 @@ void ConsumerImpl::internalGetLastMessageIdAsync(const BackoffPtr& backoff, Time
         }
     } else {
         TimeDuration next = std::min(remainTime, backoff->next());
-        if (next.total_milliseconds() <= 0) {
+        if (toMillis(next) <= 0) {
             LOG_ERROR(getName() << " Client Connection not ready for Consumer");
             callback(ResultNotConnected, MessageId());
             return;
@@ -1593,8 +1615,8 @@ void ConsumerImpl::internalGetLastMessageIdAsync(const BackoffPtr& backoff, Time
 
         auto self = shared_from_this();
         timer->async_wait([this, backoff, remainTime, timer, next, callback,
-                           self](const boost::system::error_code& ec) -> void {
-            if (ec == boost::asio::error::operation_aborted) {
+                           self](const ASIO_ERROR& ec) -> void {
+            if (ec == ASIO::error::operation_aborted) {
                 LOG_DEBUG(getName() << " Get last message id operation was cancelled, code[" << ec << "].");
                 return;
             }
@@ -1603,7 +1625,7 @@ void ConsumerImpl::internalGetLastMessageIdAsync(const BackoffPtr& backoff, Time
                 return;
             }
             LOG_WARN(getName() << " Could not get connection while getLastMessageId -- Will try again in "
-                               << next.total_milliseconds() << " ms")
+                               << toMillis(next) << " ms")
             this->internalGetLastMessageIdAsync(backoff, remainTime, timer, callback);
         });
     }
@@ -1625,8 +1647,8 @@ bool ConsumerImpl::isConnected() const { return !getCnx().expired() && state_ ==
 
 uint64_t ConsumerImpl::getNumberOfConnectedConsumer() { return isConnected() ? 1 : 0; }
 
-void ConsumerImpl::seekAsyncInternal(long requestId, SharedBuffer seek, const MessageId& seekId,
-                                     long timestamp, ResultCallback callback) {
+void ConsumerImpl::seekAsyncInternal(long requestId, SharedBuffer seek, const SeekArg& seekArg,
+                                     ResultCallback callback) {
     ClientConnectionPtr cnx = getCnx().lock();
     if (!cnx) {
         LOG_ERROR(getName() << " Client Connection not ready for Consumer");
@@ -1634,15 +1656,24 @@ void ConsumerImpl::seekAsyncInternal(long requestId, SharedBuffer seek, const Me
         return;
     }
 
-    const auto originalSeekMessageId = seekMessageId_.get();
-    seekMessageId_ = seekId;
-    duringSeek_ = true;
-    if (timestamp > 0) {
-        LOG_INFO(getName() << " Seeking subscription to " << timestamp);
-    } else {
-        LOG_INFO(getName() << " Seeking subscription to " << seekId);
+    auto expected = SeekStatus::NOT_STARTED;
+    if (!seekStatus_.compare_exchange_strong(expected, SeekStatus::IN_PROGRESS)) {
+        LOG_ERROR(getName() << " attempted to seek " << seekArg << " when the status is "
+                            << static_cast<int>(expected));
+        callback(ResultNotAllowedError);
+        return;
     }
 
+    const auto originalSeekMessageId = seekMessageId_.get();
+    if (boost::get<uint64_t>(&seekArg)) {
+        hasSoughtByTimestamp_.store(true, std::memory_order_release);
+    } else {
+        seekMessageId_ = *boost::get<MessageId>(&seekArg);
+    }
+    seekStatus_ = SeekStatus::IN_PROGRESS;
+    seekCallback_ = std::move(callback);
+    LOG_INFO(getName() << " Seeking subscription to " << seekArg);
+
     std::weak_ptr<ConsumerImpl> weakSelf{get_shared_this_ptr()};
 
     cnx->sendRequestWithId(seek, requestId)
@@ -1660,12 +1691,21 @@ void ConsumerImpl::seekAsyncInternal(long requestId, SharedBuffer seek, const Me
                 Lock lock(mutexForMessageId_);
                 lastDequedMessageId_ = MessageId::earliest();
                 lock.unlock();
+                if (getCnx().expired()) {
+                    // It's during reconnection, complete the seek future after connection is established
+                    seekStatus_ = SeekStatus::COMPLETED;
+                } else {
+                    if (!hasSoughtByTimestamp_.load(std::memory_order_acquire)) {
+                        startMessageId_ = seekMessageId_.get();
+                    }
+                    seekCallback_.release()(result);
+                }
             } else {
                 LOG_ERROR(getName() << "Failed to seek: " << result);
                 seekMessageId_ = originalSeekMessageId;
-                duringSeek_ = false;
+                seekStatus_ = SeekStatus::NOT_STARTED;
+                seekCallback_.release()(result);
             }
-            callback(result);
         });
 }
 
@@ -1694,10 +1734,11 @@ std::shared_ptr<ConsumerImpl> ConsumerImpl::get_shared_this_ptr() {
 }
 
 void ConsumerImpl::cancelTimers() noexcept {
-    boost::system::error_code ec;
+    ASIO_ERROR ec;
     batchReceiveTimer_->cancel(ec);
     checkExpiredChunkedTimer_->cancel(ec);
     unAckedMessageTrackerPtr_->stop();
+    consumerStatsBasePtr_->stop();
 }
 
 void ConsumerImpl::processPossibleToDLQ(const MessageId& messageId, ProcessDLQCallBack cb) {
@@ -1715,6 +1756,7 @@ void ConsumerImpl::processPossibleToDLQ(const MessageId& messageId, ProcessDLQCa
             ProducerConfiguration producerConfiguration;
             producerConfiguration.setSchema(config_.getSchema());
             producerConfiguration.setBlockIfQueueFull(false);
+            producerConfiguration.setBatchingEnabled(false);
             producerConfiguration.impl_->initialSubscriptionName =
                 deadLetterPolicy_.getInitialSubscriptionName();
             ClientImplPtr client = client_.lock();
@@ -1782,7 +1824,7 @@ void ConsumerImpl::processPossibleToDLQ(const MessageId& messageId, ProcessDLQCa
                         }
                         if (result != ResultOk) {
                             LOG_WARN("{" << self->topic() << "} {" << self->subscription_ << "} {"
-                                         << self->consumerName_ << "} Failed to acknowledge the message {"
+                                         << self->getConsumerName() << "} Failed to acknowledge the message {"
                                          << originMessageId
                                          << "} of the original topic but send to the DLQ successfully : "
                                          << result);
@@ -1795,7 +1837,7 @@ void ConsumerImpl::processPossibleToDLQ(const MessageId& messageId, ProcessDLQCa
                     });
                 } else {
                     LOG_WARN("{" << self->topic() << "} {" << self->subscription_ << "} {"
-                                 << self->consumerName_ << "} Failed to send DLQ message to {"
+                                 << self->getConsumerName() << "} Failed to send DLQ message to {"
                                  << self->deadLetterPolicy_.getDeadLetterTopic() << "} for message id "
                                  << "{" << originMessageId << "} : " << res);
                     cb(false);
