@@ -20,20 +20,18 @@
 #define _PULSAR_HANDLER_BASE_HEADER_
 #include <pulsar/Result.h>
 
-#include <boost/asio/deadline_timer.hpp>
+#include <boost/optional.hpp>
 #include <memory>
 #include <mutex>
 #include <string>
 
+#include "AsioTimer.h"
 #include "Backoff.h"
 #include "Future.h"
+#include "TimeUtils.h"
 
 namespace pulsar {
 
-using namespace boost::posix_time;
-using boost::posix_time::milliseconds;
-using boost::posix_time::seconds;
-
 class ClientImpl;
 using ClientImplPtr = std::shared_ptr<ClientImpl>;
 using ClientImplWeakPtr = std::weak_ptr<ClientImpl>;
@@ -42,7 +40,6 @@ using ClientConnectionPtr = std::shared_ptr<ClientConnection>;
 using ClientConnectionWeakPtr = std::weak_ptr<ClientConnection>;
 class ExecutorService;
 using ExecutorServicePtr = std::shared_ptr<ExecutorService>;
-using DeadlineTimerPtr = std::shared_ptr<boost::asio::deadline_timer>;
 
 class HandlerBase : public std::enable_shared_from_this<HandlerBase> {
    public:
@@ -55,13 +52,26 @@ class HandlerBase : public std::enable_shared_from_this<HandlerBase> {
     ClientConnectionWeakPtr getCnx() const;
     void setCnx(const ClientConnectionPtr& cnx);
     void resetCnx() { setCnx(nullptr); }
+    void setRedirectedClusterURI(const std::string& serviceUrl);
+    const std::string& getRedirectedClusterURI();
 
    protected:
+    /*
+     * tries reconnection and sets connection_ to valid object
+     * @param assignedBrokerUrl assigned broker url to directly connect to without lookup
+     */
+    void grabCnx(const boost::optional<std::string>& assignedBrokerUrl);
+
     /*
      * tries reconnection and sets connection_ to valid object
      */
     void grabCnx();
 
+    /*
+     * Schedule reconnection after backoff time
+     * @param assignedBrokerUrl assigned broker url to directly connect to without lookup
+     */
+    void scheduleReconnection(const boost::optional<std::string>& assignedBrokerUrl);
     /*
      * Schedule reconnection after backoff time
      */
@@ -90,12 +100,19 @@ class HandlerBase : public std::enable_shared_from_this<HandlerBase> {
     const std::string& topic() const { return *topic_; }
     const std::shared_ptr<std::string>& getTopicPtr() const { return topic_; }
 
+    long firstRequestIdAfterConnect() const {
+        return firstRequestIdAfterConnect_.load(std::memory_order_acquire);
+    }
+
    private:
     const std::shared_ptr<std::string> topic_;
 
+    Future<Result, ClientConnectionPtr> getConnection(const ClientImplPtr& client,
+                                                      const boost::optional<std::string>& assignedBrokerUrl);
+
     void handleDisconnection(Result result, const ClientConnectionPtr& cnx);
 
-    void handleTimeout(const boost::system::error_code& ec);
+    void handleTimeout(const ASIO_ERROR& ec, const boost::optional<std::string>& assignedBrokerUrl);
 
    protected:
     ClientImplWeakPtr client_;
@@ -127,12 +144,20 @@ class HandlerBase : public std::enable_shared_from_this<HandlerBase> {
 
     Result convertToTimeoutIfNecessary(Result result, ptime startTimestamp) const;
 
+    void setFirstRequestIdAfterConnect(long requestId) {
+        firstRequestIdAfterConnect_.store(requestId, std::memory_order_release);
+    }
+
    private:
     DeadlineTimerPtr timer_;
+    DeadlineTimerPtr creationTimer_;
 
     mutable std::mutex connectionMutex_;
     std::atomic<bool> reconnectionPending_;
     ClientConnectionWeakPtr connection_;
+    std::string redirectedClusterURI_;
+    std::atomic<long> firstRequestIdAfterConnect_{-1L};
+
     friend class ClientConnection;
     friend class PulsarFriend;
 };
