@@ -29,8 +29,8 @@
 #include <thread>
 #include <vector>
 
-#include "HttpHelper.h"
 #include "NoOpsCryptoKeyReader.h"
+#include "PulsarAdminHelper.h"
 #include "PulsarFriend.h"
 #include "SynchronizedQueue.h"
 #include "WaitUtils.h"
@@ -955,7 +955,7 @@ TEST(ConsumerTest, testGetLastMessageIdBlockWhenConnectionDisconnected) {
     auto elapsed = TimeUtils::now() - start;
 
     // getLastMessageIdAsync should be blocked until operationTimeout when the connection is disconnected.
-    ASSERT_GE(elapsed.seconds(), operationTimeout);
+    ASSERT_GE(std::chrono::duration_cast<std::chrono::seconds>(elapsed).count(), operationTimeout);
 }
 
 TEST(ConsumerTest, testRedeliveryOfDecryptionFailedMessages) {
@@ -1432,4 +1432,62 @@ TEST(ConsumerTest, testCloseAgainBeforeCloseDone) {
     ASSERT_TRUE(*done);
 }
 
+inline std::string getConsumerName(const std::string& topic) {
+    boost::property_tree::ptree root;
+    const auto error = getTopicStats(topic, root);
+    if (!error.empty()) {
+        LOG_INFO(error);
+        return {};
+    }
+    return root.get_child("subscriptions")
+        .get_child("sub")
+        .get_child("consumers")
+        .front()
+        .second.get<std::string>("consumerName");
+}
+
+TEST(ConsumerTest, testConsumerName) {
+    Client client{lookupUrl};
+    Consumer consumer;
+    ASSERT_TRUE(consumer.getConsumerName().empty());
+    const auto topic1 = "consumer-test-consumer-name-1";
+    const auto topic2 = "consumer-test-consumer-name-2";
+
+    // Default consumer name
+    ASSERT_EQ(ResultOk, client.subscribe(topic1, "sub", consumer));
+    LOG_INFO("Random consumer name: " << consumer.getConsumerName());
+    ASSERT_FALSE(consumer.getConsumerName().empty());  // a random name
+    ASSERT_EQ(consumer.getConsumerName(), getConsumerName(topic1));
+    consumer.close();
+
+    // Single-topic consumer
+    ConsumerConfiguration conf;
+    const std::string consumerName = "custom-consumer";
+    conf.setConsumerName(consumerName);
+    ASSERT_EQ(ResultOk, client.subscribe(topic1, "sub", conf, consumer));
+    ASSERT_EQ(consumerName, consumer.getConsumerName());
+    ASSERT_EQ(consumerName, getConsumerName(topic1));
+    consumer.close();
+
+    // Multi-topics consumer
+    ASSERT_EQ(ResultOk, client.subscribe(std::vector<std::string>{topic1, topic2}, "sub", conf, consumer));
+    ASSERT_EQ(consumerName, consumer.getConsumerName());
+    ASSERT_EQ(consumerName, getConsumerName(topic1));
+    ASSERT_EQ(consumerName, getConsumerName(topic2));
+
+    client.close();
+}
+
+TEST(ConsumerTest, testSNIProxyConnect) {
+    ClientConfiguration clientConfiguration;
+    clientConfiguration.setProxyServiceUrl(lookupUrl);
+    clientConfiguration.setProxyProtocol(ClientConfiguration::SNI);
+
+    Client client(lookupUrl, clientConfiguration);
+    const std::string topic = "testSNIProxy-" + std::to_string(time(nullptr));
+
+    Consumer consumer;
+    ASSERT_EQ(ResultOk, client.subscribe(topic, "test-sub", consumer));
+    client.close();
+}
 }  // namespace pulsar
