@@ -243,6 +243,8 @@ TEST_P(ProducerTest, testMaxMessageSize) {
     ASSERT_EQ(ResultMessageTooBig,
               producer.send(MessageBuilder().setContent(std::string(maxMessageSize, 'b')).build()));
 
+    ASSERT_EQ(ResultOk, producer.send(MessageBuilder().setContent(msg).build()));
+
     client.close();
 }
 
@@ -649,4 +651,36 @@ TEST(ProducerTest, testReconnectMultiConnectionsPerBroker) {
     client.close();
 }
 
+TEST(ProducerTest, testFailedToCreateNewPartitionProducer) {
+    const std::string topic =
+        "public/default/testFailedToCreateNewPartitionProducer" + std::to_string(time(nullptr));
+    std::string topicOperateUrl = adminUrl + "admin/v2/persistent/" + topic + "/partitions";
+
+    int res = makePutRequest(topicOperateUrl, "2");
+    ASSERT_TRUE(res == 204 || res == 409) << "res: " << res;
+
+    ClientConfiguration clientConf;
+    clientConf.setPartititionsUpdateInterval(1);
+    Client client(serviceUrl, clientConf);
+    ProducerConfiguration conf;
+    Producer producer;
+    client.createProducer(topic, conf, producer);
+    ASSERT_TRUE(waitUntil(std::chrono::seconds(1), [&producer]() -> bool { return producer.isConnected(); }));
+
+    PartitionedProducerImpl& partitionedProducer = PulsarFriend::getPartitionedProducerImpl(producer);
+    PulsarFriend::updatePartitions(partitionedProducer, 3);
+    std::this_thread::sleep_for(std::chrono::milliseconds(500));
+    auto& newProducer = PulsarFriend::getInternalProducerImpl(producer, 2);
+    ASSERT_FALSE(newProducer.isConnected());  // should fail with topic not found
+
+    res = makePostRequest(topicOperateUrl, "3");
+    ASSERT_TRUE(res == 204 || res == 409) << "res: " << res;
+
+    ASSERT_TRUE(
+        waitUntil(std::chrono::seconds(5), [&newProducer]() -> bool { return newProducer.isConnected(); }));
+
+    producer.close();
+    client.close();
+}
+
 INSTANTIATE_TEST_CASE_P(Pulsar, ProducerTest, ::testing::Values(true, false));
